# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - SharedUpdates Action

    @copyright: 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase
from MoinMoin.Page import Page
from MoinMoin import wikiutil
from MoinShare import getUpdatedTime, getUpdatesFromPage
from MoinSupport import escattr, get_form, getMetadata, getPathInfo, \
    writeHeaders, ActionSupport

escape = wikiutil.escape

Dependencies = ['pages']

# Action class and supporting functions.

class SharedUpdates(ActionBase, ActionSupport):

    "An update manifest retrieval dialogue requesting various parameters."

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request
        form = self.get_form()
        page = self.page

        pagename = form.get("pagename", [page.page_name])[0]

        d = {
            "buttons_html"          : buttons_html,
            "pagename_label"        : escape(_("Page name for updates")),
            "pagename"              : escattr(pagename),
            }

        return '''
<table>
    <tr>
        <td class="label"><label>%(pagename_label)s</label></td>
        <td class="content">
            <input name="pagename" type="text" size="40" value="%(pagename)s" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>
''' % d

    def do_action(self):

        "Write the syndication resource."

        write_resource(self.request)
        return 1, None

    def render_success(self, msg, msgtype=None):

        """
        Render neither 'msg' nor 'msgtype' since a resource has already been
        produced.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

def write_resource(request):

    """
    For the given 'request', write an Atom summary of updates found on the
    specified page.
    See: http://tools.ietf.org/html/rfc4287
    """

    form = get_form(request)

    pagename = form.get("pagename", [request.page.page_name])[0]

    page = Page(request, pagename)
    metadata = getMetadata(page)

    # Output summary data...

    writeHeaders(request, "application/atom+xml", metadata)
    updated = getUpdatedTime(metadata)

    # Atom output...

    # Using the page name and the page URL in the title, link and
    # subtitle.

    path_info = getPathInfo(request)
    link = "%s%s" % (request.getBaseURL(), path_info)

    d = {
        "title"     : escape(path_info[1:]),
        "link"      : escape(link),
        "href"      : escattr(link),
        "updated"   : escape(updated.as_ISO8601_datetime_string()),
        }

    # Start the output.

    request.write('''\
<?xml version="1.0" encoding="utf-8"?>\r
   <feed xmlns="http://www.w3.org/2005/Atom">\r
   <title type="text">%(title)s</title>\r
   <subtitle type="text">Shared updates published on %(link)s</subtitle>\r
   <link rel="self" href="%(href)s"/>\r
   <updated>%(updated)s</updated>\r
''' % d)

    for update in getUpdatesFromPage(page, request):

        # Get the URL that yields only the fragment.

        fragment_link = "%s?action=SharedUpdate&fragment=%s" % (update.link, update.fragment)

        download_links = []

        for mimetype in update.preferred:
            specific_link = "%s&type=%s&doit=1" % (fragment_link, mimetype)

            download_links.append('<link rel="alternate" type="%s" href="%s"/>' % (
                escattr(mimetype), escattr(specific_link)))

        d = {
            "title"             : escape(update.title),
            "fragment_link"     : escape(fragment_link),
            "download_links"    : "\r\n".join(download_links),
            "content"           : update.content and ('<content type="html">%s</content>' % escape(update.content)) or "",
            "updated"           : escape(update.updated.as_ISO8601_datetime_string()),
            }

        # Write the entry output.

        request.write('''\
<entry>\r
<title>%(title)s</title>\r
<id>%(fragment_link)s</id>\r
<updated>%(updated)s</updated>\r
%(download_links)s\r
%(content)s\r
</entry>\r
''' % d)

    # End the feed output.

    request.write('</feed>\r\n')

# Action function.

def execute(pagename, request):
    SharedUpdates(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
