# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - SharedContent macro, based on the FeedReader macro

    @copyright: 2008, 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from DateSupport import getDateTimeFromISO8601, DateTime
from MoinMoin.Page import Page
from MoinRemoteSupport import *
from MoinSupport import parseMacroArguments, getParsersForContentType, \
                        formatText, formatTextForOutputType
from MoinShare import getUpdateSources, getUpdatesFromPage, \
                      getUpdatesFromStore, getUpdateFromPart, \
                      Update, get_make_parser
from email.utils import parsedate
import xml.dom.pulldom

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

Dependencies = ["time"]

MAX_ENTRIES = 5
ATOM_NS = "http://www.w3.org/2005/Atom"

# Utility functions.

def text(element):
    nodes = []
    for node in element.childNodes:
        if node.nodeType == node.TEXT_NODE:
            nodes.append(node.nodeValue)
    return "".join(nodes)

def children(element):
    nodes = []
    for node in element.childNodes:
        nodes.append(node.toxml())
    return "".join(nodes)

def unescape(text):
    return text.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")

def linktext(element, feed_type):
    if feed_type == "rss":
        return text(element)
    else:
        return element.getAttribute("href")

def need_content(show_content, tagname):
    return show_content in ("content", "description") and tagname in ("content", "description")

# Error classes.

class FeedError(Exception):
    pass

class FeedMissingError(FeedError):
    pass

class FeedContentTypeError(FeedError):
    pass

# Feed retrieval.

def getUpdates(request, feed_url, max_entries, show_content):

    """
    Using the given 'request', retrieve from 'feed_url' up to the given number
    'max_entries' of update entries. The 'show_content' parameter can indicate
    that a "summary" is to be obtained for each update, that the "content" of
    each update is to be obtained (falling back to a summary if no content is
    provided), or no content (indicated by a false value) is to be obtained.

    A tuple of the form ((feed_type, channel_title, channel_link), updates) is
    returned.
    """

    feed_updates = []

    # Obtain the resource, using a cached version if appropriate.

    max_cache_age = int(getattr(request.cfg, "moin_share_max_cache_age", "300"))
    data = getCachedResource(request, feed_url, "MoinShare", "wiki", max_cache_age)
    if not data:
        raise FeedMissingError

    # Interpret the cached feed.

    feed = StringIO(data)
    _url, content_type, _encoding, _metadata = getCachedResourceMetadata(feed)

    if content_type not in ("application/atom+xml", "application/rss+xml", "application/xml"):
        raise FeedContentTypeError

    try:
        # Parse each node from the feed.

        channel_title = channel_link = None

        feed_type = None
        update = None
        in_source = False

        events = xml.dom.pulldom.parse(feed)

        for event, value in events:

            if not in_source and event == xml.dom.pulldom.START_ELEMENT:
                tagname = value.localName

                # Detect the feed type and items.

                if tagname == "feed" and value.namespaceURI == ATOM_NS:
                    feed_type = "atom"

                elif tagname == "rss":
                    feed_type = "rss"

                # Detect items.

                elif feed_type == "rss" and tagname == "item" or \
                    feed_type == "atom" and tagname == "entry":

                    update = Update()

                # Detect source declarations.

                elif feed_type == "atom" and tagname == "source":
                    in_source = True

                # Handle item elements.

                elif tagname == "title":
                    events.expandNode(value)
                    if update:
                        update.title = text(value)
                    else:
                        channel_title = text(value)

                elif tagname == "link":
                    events.expandNode(value)
                    if update:
                        update.link = linktext(value, feed_type)
                    else:
                        channel_link = linktext(value, feed_type)

                elif show_content and (
                    feed_type == "atom" and tagname in ("content", "summary") or
                    feed_type == "rss" and tagname == "description"):

                    events.expandNode(value)

                    # Obtain content where requested or, failing that, a
                    # summary.

                    if update and (need_content(show_content, tagname) or tagname == "summary" and not update.content):
                        if feed_type == "atom":
                            update.content_type = value.getAttribute("type") or "text"

                            # Normalise the content types and extract the
                            # content.

                            if update.content_type in ("xhtml", "application/xhtml+xml", "application/xml"):
                                update.content = children(value)
                                update.content_type = "application/xhtml+xml"
                            elif update.content_type in ("html", "text/html"):
                                update.content = text(value)
                                update.content_type = "text/html"
                            else:
                                update.content = text(value)
                                update.content_type = "text/plain"
                        else:
                            update.content_type = "text/html"
                            update.content = text(value)

                elif feed_type == "atom" and tagname == "updated" or \
                    feed_type == "rss" and tagname == "pubDate":

                    events.expandNode(value)

                    if update:
                        if feed_type == "atom":
                            value = getDateTimeFromISO8601(text(value))
                        else:
                            value = DateTime(parsedate(text(value)))
                        update.updated = value

            elif event == xml.dom.pulldom.END_ELEMENT:
                tagname = value.localName

                if feed_type == "rss" and tagname == "item" or \
                    feed_type == "atom" and tagname == "entry":

                    feed_updates.append(update)

                    update = None

                elif feed_type == "atom" and tagname == "source":
                    in_source = False

    finally:
        feed.close()

    return (feed_type, channel_title, channel_link), feed_updates

# Update formatting.

def getUpdatesForFormatting(update):

    "Get a list of updates for formatting given 'update'."

    updates = []

    # Handle multipart/alternative.

    if update.parts:
        for n, part in enumerate(update.parts):
            update_part = getUpdateFromPart(update, part, n)
            updates += getUpdatesForFormatting(update_part)
    else:
        updates.append(update)

    return updates

def getFormattedUpdate(update, request, fmt):

    """
    Return the formatted form of the given 'update' using the given 'request'
    and 'fmt'.
    """

    # NOTE: Some control over the HTML and XHTML should be exercised.

    if update.content:
        if update.content_type == "text/html" and update.message_number is not None:
            parsers = [get_make_parser(update.page, update.message_number)]
        else:
            parsers = getParsersForContentType(request.cfg, update.content_type)

        if parsers:
            for parser_cls in parsers:
                if hasattr(parser_cls, "formatForOutputType"):
                    return formatTextForOutputType(update.content, request, parser_cls, "text/html")
                else:
                    return formatText(update.content, request, fmt, parser_cls=parser_cls)
                break
        else:
            return None
    else:
        return None

def formatUpdate(update, request, fmt):

    "Format the given 'update' using the given 'request' and 'fmt'."

    result = []
    append = result.append

    updates = getUpdatesForFormatting(update)
    single = len(updates) == 1

    # Format some navigation tabs.

    if not single:
        append(fmt.div(on=1, css_class="moinshare-alternatives"))

        first = True

        for update_part in updates:
            append(fmt.url(1, "#%s" % update_part.unique_id()))
            append(fmt.text(update_part.content_type))
            append(fmt.url(0))

            first = False

        append(fmt.div(on=0))

    # Format the content.

    first = True

    for update_part in updates:

        # Encapsulate each alternative if many exist.

        if not single:
            css_class = first and "moinshare-default" or "moinshare-other"
            append(fmt.div(on=1, css_class="moinshare-alternative %s" % css_class, id=update_part.unique_id()))

        # Include the content.

        append(formatUpdatePart(update_part, request, fmt))

        if not single:
            append(fmt.div(on=0))

        first = False

    return "".join(result)

def formatUpdatePart(update, request, fmt):

    "Format the given 'update' using the given 'request' and 'fmt'."

    _ = request.getText

    result = []
    append = result.append

    # Encapsulate the content.

    append(fmt.div(on=1, css_class="moinshare-content"))
    text = getFormattedUpdate(update, request, fmt)
    if text:
        append(text)
    else:
        append(fmt.text(_("Update cannot be shown for content of type %s.") % update.content_type))
    append(fmt.div(on=0))

    return "".join(result)

# The macro itself.

def execute(macro, args):
    request = macro.request
    fmt = macro.formatter
    _ = request.getText

    source_pages = []
    show_content = None
    max_entries = None

    for arg, value in parseMacroArguments(args):
        if arg == "sources":
            source_pages.append(value)
        elif arg == "show":
            show_content = value.lower()
        elif arg == "limit":
            try:
                max_entries = int(value)
            except ValueError:
                return fmt.text(_("SharedContent: limit must be set to the maximum number of entries to be shown"))

    if not source_pages:
        return fmt.text(_("SharedContent: at least one sources page must be specified"))

    sources = {}

    for source_page in source_pages:
        sources.update(getUpdateSources(source_page, request))

    if not sources:
        return fmt.text(_("SharedContent: at least one update source must be specified"))

    show_content = show_content or False
    max_entries = max_entries or MAX_ENTRIES

    # Retrieve updates, classifying them as missing or bad and excluding them if
    # appropriate.

    updates = []
    feeds = []
    unspecified = []
    missing = []
    bad_content = []

    for source_name, source_parameters in sources.items():
        location = source_parameters.get("location")
        if not location:
            unspecified.append(source_name)
            continue

        try:
            max_entries_for_feed = int(source_parameters["limit"])
        except (KeyError, ValueError):
            max_entries_for_feed = None

        # Retrieve updates from feeds.

        if source_parameters.get("type") == "url":
            try:
                feed_info, feed_updates = getUpdates(request, location, max_entries_for_feed, show_content)
                updates += feed_updates
                feeds.append((location, feed_info))
            except FeedMissingError:
                missing.append(location)
            except FeedContentTypeError:
                bad_content.append(location)

        # Retrieve updates from pages.

        elif source_parameters.get("type") == "page":
            page = Page(request, location)
            updates += getUpdatesFromPage(page, request)

            # Build feed-equivalent information for the update source.

            feeds.append((
                page.url(request, {"action" : "SharedUpdates", "doit" : "1"}), (
                    "internal", _("Updates from page %s") % location,
                    page.url(request)
                    )
                ))

        # Retrieve updates from message stores.

        elif source_parameters.get("type") == "store":
            page = Page(request, location)
            updates += getUpdatesFromStore(page, request)

            # Build feed-equivalent information for the update source.

            feeds.append((
                page.url(request, {"action" : "SharedUpdates", "store" : "1", "doit" : "1"}), (
                    "internal", _("Updates from message store on page %s") % location,
                    page.url(request)
                    )
                ))

    # Prepare the output.

    output = []
    append = output.append

    # Show the updates.

    if not show_content:
        append(fmt.bullet_list(on=1))

    # NOTE: Permit configurable sorting.

    updates.sort()
    updates.reverse()

    # Truncate the number of updates to the maximum number.

    updates = updates[:max_entries]

    for update in updates:

        # Emit content where appropriate.

        if show_content:
            append(fmt.div(on=1, css_class="moinshare-update"))

            append(formatUpdate(update, request, fmt))

            append(fmt.div(on=1, css_class="moinshare-date"))
            append(fmt.text(str(update.updated)))
            append(fmt.div(on=0))

            append(fmt.div(on=0))

        # Or emit title and link information for items.

        elif update.title and update.link:
            append(fmt.listitem(on=1, css_class="moinshare-update"))
            append(fmt.url(on=1, href=update.link))
            append(fmt.icon('www'))
            append(fmt.text(" " + update.title))
            append(fmt.url(on=0))
            append(fmt.listitem(on=0))

    if not show_content:
        append(fmt.bullet_list(on=0))

    # Show the feeds.

    for feed_url, (feed_type, channel_title, channel_link) in feeds:
        if channel_title and channel_link:
            append(fmt.paragraph(on=1, css_class="moinshare-feed"))
            append(fmt.url(on=1, href=channel_link))
            append(fmt.text(channel_title))
            append(fmt.url(on=0))
            append(fmt.text(" "))
            append(fmt.url(on=1, href=feed_url))
            append(fmt.icon('rss'))
            append(fmt.url(on=0))
            append(fmt.paragraph(on=0))

    # Show errors.

    for feed_url in missing:
        append(fmt.paragraph(on=1, css_class="moinshare-missing-feed-error"))
        append(fmt.text(_("SharedContent: updates could not be retrieved for %s") % feed_url))
        append(fmt.paragraph(on=0))

    for feed_url in bad_content:
        append(fmt.paragraph(on=1, css_class="moinshare-content-type-feed-error"))
        return fmt.text(_("SharedContent: updates for %s were not provided in Atom or RSS format") % feed_url)
        append(fmt.paragraph(on=0))

    return ''.join(output)

# vim: tabstop=4 expandtab shiftwidth=4
